from dataclasses import field
from datetime import datetime
from typing import List, Optional

from pydantic.dataclasses import dataclass



@dataclass
class TypeA:
    one: str
    two: float


@dataclass
class TypeB(TypeA):
    one: str
    three: bool = field(default=True)


@dataclass
class TypeC(TypeB):
    four: List[datetime] = field(default_factory=list, metadata={"format": "%d %B %Y %H:%M"})
    any: Optional[object] = field(default=None, metadata={"type": "Wildcard"})



from dataclasses import field
from dataclasses import is_dataclass
from typing import Any
from typing import Callable
from typing import Dict
from typing import Generic
from typing import List
from typing import Optional
from typing import Type
from typing import TypeVar
from xml.etree.ElementTree import QName

from pydantic.dataclasses import dataclass
from pydantic.validators import _VALIDATORS
from xsdata.formats.dataclass.compat import class_types
from xsdata.formats.dataclass.compat import Dataclasses
from xsdata.formats.dataclass.models.elements import XmlType
from xsdata.models.datatype import XmlDate
from xsdata.models.datatype import XmlDateTime
from xsdata.models.datatype import XmlDuration
from xsdata.models.datatype import XmlPeriod
from xsdata.models.datatype import XmlTime


T = TypeVar("T", bound=object)


class Config:
    arbitrary_types_allowed = True


@dataclass(config=Config)
class AnyElement:
    """
    Generic model to bind xml document data to wildcard fields.

    :param qname: The element's qualified name
    :param text: The element's text content
    :param tail: The element's tail content
    :param children: The element's list of child elements.
    :param attributes: The element's key-value attribute mappings.
    """

    qname: Optional[str] = field(default=None)
    text: Optional[str] = field(default=None)
    tail: Optional[str] = field(default=None)
    children: List[object] = field(
        default_factory=list, metadata={"type": XmlType.WILDCARD}
    )
    attributes: Dict[str, str] = field(
        default_factory=dict, metadata={"type": XmlType.ATTRIBUTES}
    )


@dataclass(config=Config)
class DerivedElement(Generic[T]):
    """
    Generic model wrapper for type substituted elements.

    Example: eg. <b xsi:type="a">...</b>

    :param qname: The element's qualified name
    :param value: The wrapped value
    :param type: The real xsi:type
    """

    qname: str
    value: T
    type: Optional[str] = None


class Pydantic(Dataclasses):
    @property
    def any_element(self) -> Type:
        return AnyElement

    @property
    def derived_element(self) -> Type:
        return DerivedElement

    def is_model(self, obj: Any) -> bool:
        clazz = obj if isinstance(obj, type) else type(obj)
        if is_dataclass(clazz) and hasattr(clazz, "__pydantic_model__"):
            clazz.__pydantic_model__.update_forward_refs()  # type: ignore
            return True

        return False


class_types.register("pydantic", Pydantic())


def make_validators(tp: Type, factory: Callable) -> List[Callable]:
    def validator(value: Any) -> Any:

        if isinstance(value, tp):
            return value

        if isinstance(value, str):
            return factory(value)

        raise ValueError

    return [validator]


_VALIDATORS.extend(
    [
        (XmlDate, make_validators(XmlDate, XmlDate.from_string)),
        (XmlDateTime, make_validators(XmlDateTime, XmlDateTime.from_string)),
        (XmlTime, make_validators(XmlTime, XmlTime.from_string)),
        (XmlDuration, make_validators(XmlDuration, XmlDuration)),
        (XmlPeriod, make_validators(XmlPeriod, XmlPeriod)),
        (QName, make_validators(QName, QName)),
    ]
)



from typing import Dict

from xsdata.formats.dataclass.filters import Filters
from xsdata.formats.dataclass.generator import DataclassGenerator
from xsdata.models.config import GeneratorConfig
from xsdata.utils.text import stop_words

stop_words.update(("schema", "validate"))


class PydanticGenerator(DataclassGenerator):
    """Python pydantic dataclasses code generator."""

    @classmethod
    def init_filters(cls, config: GeneratorConfig) -> Filters:
        return PydanticFilters(config)


class PydanticFilters(Filters):
    @classmethod
    def build_import_patterns(cls) -> Dict[str, Dict]:
        patterns = super().build_import_patterns()
        patterns.update(
            {
                "dataclasses": {"field": [" = field("]},
                "pydantic.dataclasses": {"dataclass": ["@dataclass"]},
            }
        )

        return {key: patterns[key] for key in sorted(patterns)}

    @classmethod
    def filter_metadata(cls, data: Dict) -> Dict:
        data = super().filter_metadata(data)
        data.pop("min_length", None)
        data.pop("max_length", None)
        return data



from dataclasses import dataclass
from dataclasses import field
from typing import Callable

from xsdata.formats.dataclass import context
from xsdata.formats.dataclass import parsers
from xsdata.formats.dataclass import serializers
from xsdata.utils.constants import return_input


class XmlContext(context.XmlContext):
    """Pydantic dataclasses ready xml context instance."""

    def __init__(
        self,
        element_name_generator: Callable = return_input,
        attribute_name_generator: Callable = return_input,
    ):
        super().__init__(element_name_generator, attribute_name_generator, "pydantic")


@dataclass
class XmlParser(parsers.XmlParser):
    context: XmlContext = field(default_factory=XmlContext)


@dataclass
class XmlSerializer(serializers.XmlSerializer):
    context: XmlContext = field(default_factory=XmlContext)


@dataclass
class JsonParser(parsers.JsonParser):
    context: XmlContext = field(default_factory=XmlContext)


@dataclass
class JsonSerializer(serializers.JsonSerializer):
    context: XmlContext = field(default_factory=XmlContext)


@dataclass
class UserXmlParser(parsers.UserXmlParser):
    context: XmlContext = field(default_factory=XmlContext)



from unittest import TestCase

from xsdata.formats.dataclass.compat import class_types

from xsdata_pydantic.compat import AnyElement
from xsdata_pydantic.compat import DerivedElement
from xsdata_pydantic.compat import Pydantic


class PydanticTests(TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.class_type = class_types.get_type("pydantic")

    def test_class_type(self):
        self.assertIsInstance(self.class_type, Pydantic)

    def test_property_any_element(self):
        self.assertIs(self.class_type.any_element, AnyElement)

    def test_property_derived_element(self):
        self.assertIs(self.class_type.derived_element, DerivedElement)
